
import 'package:benchmark_harness/benchmark_harness.dart';
import 'package:graphlib/graphlib.dart';

typedef BaseGraph graphFactory();

int SIZE = 200;

final all_graphs = [
  {'name': 'Graph', 'f': () => new Graph()},
  {'name': 'Digraph', 'f': () => new Digraph()},
  {'name': 'CGraph', 'f': () => new CGraph()},
  {'name': 'CDigraph', 'f': () => new CDigraph()}
];

final graphs = [
  {'name': 'Graph', 'f': () => new Graph()},
  {'name': 'CGraph', 'f': () => new CGraph()},
];

final digraphs = [
  {'name': 'Digraph', 'f': () => new Digraph()},
  {'name': 'CDigraph', 'f': () => new CDigraph()}
];

abstract class GraphBenchmark extends BenchmarkBase {
  final graphFactory f;
  BaseGraph g;
  GraphBenchmark(name, this.f) : super(name);

  void run();

  void setup() {
    g = f();
    for (int i = 0; i < SIZE; ++i) {
      g.addNode(i);
      for (int j = i ~/ 2; j > 0; j = j ~/ 2) {
        g.addEdge(null, j, i);
      }
    }
  }

  void teardown() {
    g = null;
  }
}

class CopyBenchmark extends GraphBenchmark {
  CopyBenchmark(name, graphFactory f) : super("$name.copy", f);

  static void main(List<Map> G) {
    for (var m in G) new CopyBenchmark(m['name'], m['f']).report();
  }

  void run() {
    g.copy();
  }
}

class FilterNodesBenchmark extends GraphBenchmark {
  Iterable us;
  FilterNodesBenchmark(name, graphFactory f) : super("$name.filterNodes", f);

  static void main(List<Map> G) {
    for (var m in G) new FilterNodesBenchmark(m['name'], m['f']).report();
  }

  void setup() {
    super.setup();
    us = g.nodes().where((u) { return (u % 2) != 0; }).toList();
  }

  void run() {
    g.filterNodes(nodesFromList(us));
  }
}

class AddNodeBenchmark extends GraphBenchmark {
  AddNodeBenchmark(name, graphFactory f) : super("$name.addNode", f);

  static void main(List<Map> G) {
    for (var m in G) new AddNodeBenchmark(m['name'], m['f']).report();
  }

  void run() {
    g.addNode();
  }
}

class AddDelNodeBenchmark extends GraphBenchmark {
  AddDelNodeBenchmark(name, graphFactory f) : super("$name.addDelNode", f);

  static void main(List<Map> G) {
    for (var m in G) new AddDelNodeBenchmark(m['name'], m['f']).report();
  }

  void run() {
    g.addNode(-1);
    g.delNode(-1);
  }
}

class AddEdgeBenchmark extends GraphBenchmark {
  AddEdgeBenchmark(name, graphFactory f) : super("$name.addEdge", f);

  static void main(List<Map> G) {
    for (var m in G) new AddEdgeBenchmark(m['name'], m['f']).report();
  }

  void run() {
//    var u1 = g.addNode();
//    var u2 = g.addNode();
    g.addEdge(null, 1, 2);
  }
}

class AddDelEdgeBenchmark extends GraphBenchmark {
  AddDelEdgeBenchmark(name, graphFactory f) : super("$name.addDelEdge", f);

  static void main(List<Map> G) {
    for (var m in G) new AddDelEdgeBenchmark(m['name'], m['f']).report();
  }

  void run() {
    var u = g.addEdge(null, 1, 2);
    g.delEdge(u);
  }
}

/* Algorithm benchmarks */

class ComponentsBenchmark extends GraphBenchmark {
  ComponentsBenchmark(name, graphFactory f) : super("$name.components", f);

  static void main(List<Map> G) {
    for (var m in G) new ComponentsBenchmark(m['name'], m['f']).report();
  }

  void run() {
    components(g);
  }
}

class DijkstraAllBenchmark extends GraphBenchmark {
  DijkstraAllBenchmark(name, graphFactory f) : super("$name.dijkstraAll", f);

  static void main(List<Map> G) {
    for (var m in G) new DijkstraAllBenchmark(m['name'], m['f']).report();
  }

  void run() {
    dijkstraAll(g);
  }
}

class TarjanBenchmark extends GraphBenchmark {
  TarjanBenchmark(name, graphFactory f) : super("$name.tarjan", f);

  static void main(List<Map> G) {
    for (var m in G) new TarjanBenchmark(m['name'], m['f']).report();
  }

  void run() {
    tarjan(g);
  }
}

class FloydWarshallBenchmark extends GraphBenchmark {
  FloydWarshallBenchmark(name, graphFactory f) : super("$name.floydWarshall", f);

  static void main(List<Map> G) {
    for (var m in G) new FloydWarshallBenchmark(m['name'], m['f']).report();
  }

  void run() {
    floydWarshall(g);
  }
}

class TopsortBenchmark extends GraphBenchmark {
  TopsortBenchmark(name, graphFactory f) : super("$name.topsort", f);

  static void main(List<Map> G) {
    for (var m in G) new TopsortBenchmark(m['name'], m['f']).report();
  }

  void run() {
    topsort(g);
  }
}

main() {
  CopyBenchmark.main(all_graphs);
  FilterNodesBenchmark.main(all_graphs);
  AddNodeBenchmark.main(all_graphs);
  AddDelNodeBenchmark.main(all_graphs);
  AddEdgeBenchmark.main(all_graphs);
  AddDelEdgeBenchmark.main(all_graphs);
  SIZE = 50;
  DijkstraAllBenchmark.main(all_graphs);
  ComponentsBenchmark.main(graphs);
  TarjanBenchmark.main(digraphs);
  FloydWarshallBenchmark.main(all_graphs);
  TopsortBenchmark.main(digraphs);
}
