part of graphlib.alg;

//var PriorityQueue = require("cp-data").PriorityQueue;

//module.exports = dijkstra;

/**
 * This function is an implementation of [Dijkstra's algorithm](http://en.wikipedia.org/wiki/Dijkstra%27s_algorithm)
 * which finds the shortest path from [source] to all other nodes in [g]. This
 * function returns a map of `u -> { distance, predecessor }`. The distance
 * property holds the sum of the weights from [source] to [u] along the
 * shortest path or `double.INFINITY` if there is no path from
 * [source]. The predecessor property can be used to walk the individual
 * elements of the path from [source] to [u] in reverse order.
 *
 * This function takes an optional [weightFunc] which returns the
 * weight of the edge `e`. If no weightFunc is supplied then each edge is
 * assumed to have a weight of 1. This function throws an [Exception] if any
 * of the traversed edges have a negative edge weight.
 *
 * This function takes an optional [incidentFunc] which returns the ids of
 * all edges incident to the node `u` for the purposes of shortest path
 * traversal. By default this function uses the `g.outEdges` for Digraphs and
 * `g.incidentEdges` for Graphs.
 *
 * This function takes `O((|E| + |V|) * log |V|)` time.
 *
 * img(src="https://cpettitt.github.io/project/graphlib/latest/doc/static/dijkstra-source.png")
 *
 *     weight(e) { return digraph.edge(e); }
 *
 *     alg.dijkstra(digraph, "A", weight);
 *     // => { A: { 'distance': 0 },
 *     //      B: { 'distance': 6, 'predecessor': 'C' },
 *     //      C: { 'distance': 4, 'predecessor': 'A' },
 *     //      D: { 'distance': 2, 'predecessor': 'A' },
 *     //      E: { 'distance': 8, 'predecessor': 'F' },
 *     //      F: { 'distance': 4, 'predecessor': 'D' } }
 */
Map dijkstra(BaseGraph g, source, [weightFunc(e)=null, incidentFunc(u)=null]) {
  var results = {},
      pq = new PriorityQueue();
  var u, uEntry;

  updateNeighbors(e) {
    var incidentNodes = g.incidentNodes(e),
        v = incidentNodes[0] != u ? incidentNodes[0] : incidentNodes[1],
        vEntry = results[v],
        weight = weightFunc(e),
        distance = uEntry['distance'] + weight;

    if (weight < 0) {
      throw new Exception("dijkstra does not allow negative edge weights. Bad edge: $e Weight: $weight");
    }

    if (distance < vEntry['distance']) {
      vEntry['distance'] = distance;
      vEntry['predecessor'] = u;
      pq.decrease(v, distance);
    }
  }

  if (weightFunc == null) {
    weightFunc = (_) { return 1; };
  }
  if (incidentFunc == null) {
    incidentFunc = (g.isDirected() ? (u) { return (g as Digraph).outEdges(u); } : (u) { return g.incidentEdges(u); });
  }

  g.eachNode((u, _) {
    var distance = u == source ? 0 : double.INFINITY;
    results[u] = { 'distance': distance };
    pq.add(u, distance);
  });

  while (pq.size() > 0) {
    u = pq.removeMin();
    uEntry = results[u];
    if (uEntry['distance'] == double.INFINITY) {
      break;
    }

    incidentFunc(u).forEach(updateNeighbors);
  }

  return results;
}
