part of graphlib.alg;

/**
 * This function finds the shortest path from each node to every other
 * reachable node in the graph. It is similar to [dijkstra], but
 * instead of returning a single-source array, it returns a mapping of
 * of `source -> dijksta(g, source, weightFunc, incidentFunc)`.
 *
 * This function takes an optional [weightFunc] which returns the
 * weight of the edge `e`. If no weightFunc is supplied then each edge is
 * assumed to have a weight of 1. This function throws an [Exception if any
 * of the traversed edges have a negative edge weight.
 *
 * This function takes an optional [incidentFunc] which returns the ids of
 * all edges incident to the node `u` for the purposes of shortest path
 * traversal. By default this function uses the `outEdges` function on the
 * supplied graph.
 *
 * This function takes `O(|V| * (|E| + |V|) * log |V|)` time.
 *
 * img(src="https://cpettitt.github.io/project/graphlib/latest/doc/static/dijkstra-source.png")
 *
 *     weight(e) { return digraph.edge(e); }
 *
 *     alg.dijkstraAll(digraph, weight);
 *     // => { A:
 *     //       { A: { 'distance': 0 },
 *     //         B: { 'distance': 6, 'predecessor': 'C' },
 *     //         C: { 'distance': 4, 'predecessor': 'A' },
 *     //         D: { 'distance': 2, 'predecessor': 'A' },
 *     //         E: { 'distance': 8, 'predecessor': 'F' },
 *     //         F: { 'distance': 4, 'predecessor': 'D' } },
 *     //      B:
 *     //       { A: { 'distance': Infinity },
 *     //         B: { 'distance': 0 },
 *     //         C: { 'distance': Infinity },
 *     //         D: { 'distance': Infinity },
 *     //         E: { 'distance': 6, 'predecessor': 'B' },
 *     //         F: { 'distance': Infinity } },
 *     //      C: { ... },
 *     //      D: { ... },
 *     //      E: { ... },
 *     //      F: { ... } }
 */
Map dijkstraAll(BaseGraph g, [weightFunc(e)=null, incidentFunc(u)=null]) {
  var results = {};
  g.eachNode((u, _) {
    results[u] = dijkstra(g, u, weightFunc, incidentFunc);
  });
  return results;
}
