part of graphlib.alg;
//module.exports = floydWarshall;

/**
 * This function is an implementation of the [Floyd-Warshall algorithm](https://en.wikipedia.org/wiki/Floyd-Warshall_algorithm),
 * which finds the shortest path from each node to every other reachable node
 * in the graph. It is similar to [dijkstraAll], but it handles negative
 * edge weights and is more efficient for some types of graphs. This function
 * returns a map of `source -> { target -> { distance, predecessor }`. The
 * distance property holds the sum of the weights from `source` to `target`
 * along the shortest path of `double.INFINITY` if there is no path
 * from `source`. The predecessor property can be used to walk the individual
 * elements of the path from `source` to `target` in reverse order.
 *
 * This function takes an optional [weightFunc] which returns the
 * weight of the edge `e`. If no weightFunc is supplied then each edge is
 * assumed to have a weight of 1.
 *
 * This function takes an optional [incidentFunc] which returns the ids of
 * all edges incident to the node `u` for the purposes of shortest path
 * traversal. By default this function uses the `outEdges` function on the
 * supplied graph.
 *
 * This algorithm takes `O(|V|^3)` time.
 *
 * img(src="https://cpettitt.github.io/project/graphlib/latest/doc/static/dijkstra-source.png")
 *
 *     weight(e) { return digraph.edge(e); }
 *
 *     alg.floydWarshall(digraph, weight);
 *     // => { A:
 *     //       { A: { 'distance': 0 },
 *     //         B: { 'distance': 6, 'predecessor': 'C' },
 *     //         C: { 'distance': 4, 'predecessor': 'A' },
 *     //         D: { 'distance': 2, 'predecessor': 'A' },
 *     //         E: { 'distance': 8, 'predecessor': 'F' },
 *     //         F: { 'distance': 4, 'predecessor': 'D' } },
 *     //      B:
 *     //       { A: { 'distance': Infinity },
 *     //         B: { 'distance': 0 },
 *     //         C: { 'distance': Infinity },
 *     //         D: { 'distance': Infinity },
 *     //         E: { 'distance': 6, 'predecessor': 'B' },
 *     //         F: { 'distance': Infinity } },
 *     //      C: { ... },
 *     //      D: { ... },
 *     //      E: { ... },
 *     //      F: { ... } }
 */
Map floydWarshall(BaseGraph g, [weightFunc(e)=null, incidentFunc(u)=null]) {
  var results = {},
      nodes = g.nodes();

  if (weightFunc == null) {
    weightFunc = (_) { return 1; };
  }
  if (incidentFunc == null) {
    incidentFunc = (g.isDirected() ? (u) { return (g as Digraph).outEdges(u); } : (u) { return g.incidentEdges(u); });
  }

  nodes.forEach((u) {
    results[u] = {};
    results[u][u] = { 'distance': 0 };
    nodes.forEach((v) {
      if (u != v) {
        results[u][v] = { 'distance': double.INFINITY };
      }
    });
    incidentFunc(u).forEach((e) {
      var incidentNodes = g.incidentNodes(e),
          v = incidentNodes[0] != u ? incidentNodes[0] : incidentNodes[1],
          d = weightFunc(e);
      if (d < results[u][v]['distance']) {
        results[u][v] = { 'distance': d, 'predecessor': u };
      }
    });
  });

  nodes.forEach((k) {
    var rowK = results[k];
    nodes.forEach((i) {
      var rowI = results[i];
      nodes.forEach((j) {
        var ik = rowI[k];
        var kj = rowK[j];
        var ij = rowI[j];
        var altDistance = ik['distance'] + kj['distance'];
        if (altDistance < ij['distance']) {
          ij['distance'] = altDistance;
          ij['predecessor'] = kj['predecessor'];
        }
      });
    });
  });

  return results;
}
