part of graphlib.alg;

//var Graph = require("../Graph"),
//    PriorityQueue = require("cp-data").PriorityQueue;
//
//module.exports = prim;

/**
 * [Prim's algorithm](https://en.wikipedia.org/wiki/Prim's_algorithm) takes
 * a connected undirected graph and generates a [minimum spanning tree](https://en.wikipedia.org/wiki/Minimum_spanning_tree).
 * This function returns the minimum spanning tree as an undirected graph.
 * This algorithm is derived from the description in "Introduction to
 * Algorithms", Third Edition, Cormen, et al., Pg 634.
 *
 * This function takes a [weightFunc] which returns the weight of the edge
 * `e`. It throws an [Exception] if the graph is not connected.
 *
 * This function takes `O(|E| log |V|)` time.
 *
 * img(src="https://cpettitt.github.io/project/graphlib/latest/doc/static/prim-source.png")
 *
 *     weight(e) { return graph.edge(e); }
 *     alg.prim(graph, weight);
 *
 * img(src="https://cpettitt.github.io/project/graphlib/latest/doc/static/prim-result.png")
 */
Graph prim(BaseGraph g, weightFunc(e)) {
  var result = new Graph(),
      parents = {},
      pq = new PriorityQueue(),
      u;

  updateNeighbors(e) {
    var incidentNodes = g.incidentNodes(e),
        v = incidentNodes[0] != u ? incidentNodes[0] : incidentNodes[1],
        pri = pq.priority(v);
    if (pri != null) {
      var edgeWeight = weightFunc(e);
      if (edgeWeight < pri) {
        parents[v] = u;
        pq.decrease(v, edgeWeight);
      }
    }
  }

  if (g.order() == 0) {
    return result;
  }

  g.eachNode((u, _) {
    pq.add(u, double.INFINITY);
    result.addNode(u);
  });

  // Start from an arbitrary node
  pq.decrease(g.nodes()[0], 0);

  var init = false;
  while (pq.size() > 0) {
    u = pq.removeMin();
    if (parents.containsKey(u)) {
      result.addEdge(null, u, parents[u]);
    } else if (init) {
      throw new Exception("Input graph is not connected: $g");
    } else {
      init = true;
    }

    g.incidentEdges(u).forEach(updateNeighbors);
  }

  return result;
}
