part of graphlib.alg;

//module.exports = tarjan;

/**
 * This function is an implementation of [Tarjan's algorithm](http://en.wikipedia.org/wiki/Tarjan's_strongly_connected_components_algorithm)
 * which finds all [strongly connected components](http://en.wikipedia.org/wiki/Strongly_connected_component)
 * in the directed graph [g]. Each strongly connected component is composed
 * of nodes that can reach all other nodes in the component via directed edges.
 * A strongly connected component can consist of a single node if that node
 * cannot both reach and be reached by any other specific node in the graph.
 * Components of more than one node are guaranteed to have at least one cycle.
 *
 * This function returns an array of components. Each component is itself an
 * array that contains the ids of all nodes in the component.
 *
 * img(src="https://cpettitt.github.io/project/graphlib/latest/doc/static/tarjan-source.png")
 *
 *     alg.tarjan(digraph);
 *     // => [ [ 'F', 'G' ],
 *     //      [ 'H', 'D', 'C' ],
 *     //      [ 'E', 'B', 'A' ] ]
 */
List tarjan(Digraph g) {
  if (!g.isDirected()) {
    throw new Exception("tarjan can only be applied to a directed graph. Bad input: $g");
  }

  var index = 0,
      stack = [],
      visited = {}, // node id -> { onStack, lowlink, index }
      results = [];

  dfs(u) {
    var entry = visited[u] = {
      'onStack': true,
      'lowlink': index,
      'index': index++
    };
    stack.add(u);

    g.successors(u).forEach((v) {
      if (!(visited.containsKey(v))) {
        dfs(v);
        entry['lowlink'] = Math.min(entry['lowlink'], visited[v]['lowlink']);
      } else if (visited[v]['onStack']) {
        entry['lowlink'] = Math.min(entry['lowlink'], visited[v]['index']);
      }
    });

    if (entry['lowlink'] == entry['index']) {
      var cmpt = [],
          v;
      do {
        v = stack.removeLast();
        visited[v]['onStack'] = false;
        cmpt.add(v);
      } while (u != v);
      results.add(cmpt);
    }
  }

  g.nodes().forEach((u) {
    if (!(visited.containsKey(u))) {
      dfs(u);
    }
  });

  return results;
}
