part of graphlib.alg;

//module.exports = topsort;
//topsort.CycleException = CycleException;

/**
 * An implementation of [topological sorting](https://en.wikipedia.org/wiki/Topological_sorting).
 *
 * Given a [Digraph] [g] this function returns an array of nodes such that
 * for each edge `u -> v`, `u` appears before `v` in the array. If the graph
 * has a cycle it is impossible to generate such a list and [CycleException]
 * is thrown.
 *
 * Takes `O(|V| + |E|)` time.
 *
 * img(src="https://cpettitt.github.io/project/graphlib/latest/doc/static/topsort-source.png")
 *
 *     alg.topsort(digraph);
 *     // => [ 1, 2, 3, 4 ]
 *     // OR
 *     // => [ 1, 3, 2, 4 ]
 */
List topsort(Digraph g) {
  if (!g.isDirected()) {
    throw new Exception("topsort can only be applied to a directed graph. Bad input: $g");
  }

  var visited = {};
  var stack = {};
  var results = [];

  visit(node) {
    if (stack.containsKey(node)) {
      throw new CycleException();
    }

    if (!(visited.containsKey(node))) {
      stack[node] = true;
      visited[node] = true;
      g.predecessors(node).forEach((pred) {
        visit(pred);
      });
      stack.remove(node);
      results.add(node);
    }
  }

  var sinks = g.sinks();
  if (g.order() != 0 && sinks.length == 0) {
    throw new CycleException();
  }

  g.sinks().forEach((sink) {
    visit(sink);
  });

  return results;
}

class CycleException extends Error {
  toString() => "Graph has at least one cycle";
}
