part of graphlib;

abstract class BaseGraph {
  /** The value assigned to the graph itself. */
  var _value = null;

  /** Map of node id -> { id, value } */
  Map _nodes = {};

  /** Map of edge id -> { id, u, v, value } */
  Map _edges = {};

  /** Used to generate a unique id in the graph. */
  int _nextId = 0;

  //BaseGraph();

  /**
   * Number of nodes.
   *
   *     graph.order();
   *     // => 0
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *
   *     graph.order();
   *     // => 2
   */
  int order() {
    return this._nodes.length;
  }

  /**
   * Number of edges.
   *
   *     graph.size();
   *     // => 0
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *     graph.addEdge(null, 1, 2);
   *
   *     graph.size();
   *     // => 1
   */
  int size() {
    return this._edges.length;
  }

  /**
   * Provides a mechansim to get and set some user-defined information on
   * the graph object itself. With no arguments [graph] returns the currently
   * set value. With one argument [graph] replaces the current value on the
   * graph with [value]. The initial value for the graph is `null`.
   *
   *     graph.graph();
   *     // => undefined
   *
   *     graph.graph("Some graph value");
   *     graph.graph();
   *     // => "Some graph value"
   */
  graph([value=null]) {
    if (value == null) {
      return _value;
    }
    _value = value;
  }
//  Object get graph => _value;
//  void set graph(Object value) {
//    this._value = value;
//  }

  /**
   * Returns `true` if the node with the id [u] is a member of this graph and
   * `false` if not.
   *
   *     graph.hasNode(1);
   *     // => false
   *
   *     graph.addNode(1);
   *     graph.hasNode(1);
   *     // => true
   */
  bool hasNode(u) {
    return this._nodes.containsKey(u);
  }

  /**
   * Provides a mechanism to get and set some user-defined information on an
   * individual node. With one argument this function returns the value
   * currently assigned to the node with the id [u]. With two arguments this
   * function replaces the current value of the node [u] with [value]. The
   * initial value for nodes is `null`.
   *
   * If there is no node [u] in this graph this function will throw an
   * [Exception].
   *
   *     graph.addNode(1);
   *     graph.node(1);
   *     // => undefined
   *
   *     graph.node(1, "Some node value");
   *     graph.node(1);
   *     // => "Some node value"
   *
   *     graph.node(2);
   *     // throws an Exception
   */
  node(u, [value=null]) {
    var node = this._strictGetNode(u);
    if (value == null) {
      return node['value'];
    }
    node['value'] = value;
  }

  /**
   * Returns the ids of all nodes in this graph. Use [node] to get the value
   * for a specific node.
   *
   *     graph.nodes();
   *     // => []
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *
   *     graph.nodes()
   *     // => [1, 2]
   */
  List nodes() {
    var nodes = [];
    this.eachNode((id, val) { nodes.add(id); });
    return nodes;
  }

  /**
   * Applies the function `f(u, value)` to each node in this graph in
   * arbitrary order, where `u` is the id of the node and `value` is its
   * currently assigned value.
   *
   *     graph.addNode(1, "label-1");
   *     graph.addNode(2, "label-2");
   *
   *     var results = {};
   *     graph.eachNode((u, value) {
   *         results[u] = value;
   *     });
   *
   *     results
   *     // => { 1: "label-1", 2: "label-2" }
   */
  void eachNode(func(id, val)) {
    for (var k in this._nodes.keys.toList()) {
      var node = this._nodes[k];
      func(node['id'], node['value']);
    }
  }

  /**
   * Returns `true` if an edge with the id [e] exists in this graph or
   * `false` if not.
   *
   *     graph.hasEdge("A");
   *     // => false
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *     graph.addEdge("A", 1, 2);
   *     graph.hasEdge("A");
   *     // => true
   */
  bool hasEdge(e) {
    return this._edges.containsKey(e);
  }

  /**
   * Provides a mechanism to get and set some user-defined information on an
   * individual edge. With one argument this function returns the value
   * currently assigned to the edge with the id [e]. With two arguments this
   * function replaces the current value of the edge [e] with [value]. The
   * initial value for edges is `null`.
   *
   * If there is no edge [e] in the graph this function will throw an
   * [Exception].
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *     graph.edge("A", 1, 2);
   *     // => undefined
   *
   *     graph.edge("A", "Some edge value");
   *     graph.edge("A");
   *     // => "Some edge value"
   *
   *     graph.edge("B");
   *     // throws an Exception
   */
  edge(e, [value=null]) {
    var edge = this._strictGetEdge(e);
    if (value == null) {
      return edge['value'];
    }
    edge['value'] = value;
  }

  /**
   * Returns the ids of all edges in this graph. Use [edge] to get the value
   * for a specific edge.
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *     graph.addEdge("A", 1, 2);
   *     graph.addEdge("B", 1, 2);
   *     graph.edges();
   *     // => ["A", "B"]
   */
  List edges() {
    var es = [];
    this.eachEdge((id, u, v, val) { es.add(id); });
    return es;
  }

  /**
   * Applies the function `f(e, u, v, value)` to each edge in the graph in
   * arbitrary order, where `e` is the edge's id, `u` and `v` are the incident
   * node ids, and `value` it the value assigned to the edge.
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *     graph.addEdge("A", 1, 2, "A-label");
   *     graph.addEdge("B", 2, 1, "B-label");
   *
   *     var results = {};
   *     graph.eachEdge((e, u, v, label) {
   *         results[e] = "U: $u V: $v L: $label";
   *     });
   *
   *     results
   *     // => { "A": "U: 1 V: 2 L: A-label",
   *     //      "B": "U: 2 V: 1 L: B-label" }
   */
  void eachEdge(func(id, u, v, val)) {
    for (var k in this._edges.keys.toList()) {
      var edge = this._edges[k];
      func(edge['id'], edge['u'], edge['v'], edge['value']);
    }
  }

  /**
   * Returns the nodes that are a part of the edge [e] in a 2-element array.
   * There is no significance to the order in which the nodes appear in the
   * array.
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *     graph.addEdge("A", 1, 2);
   *     graph.incidentNodes("A");
   *     // => [ 1, 2 ]
   */
  List incidentNodes(e) {
    var edge = this._strictGetEdge(e);
    return [edge['u'], edge['v']];
  }

  /**
   * Adds a new node with the id [u] to this graph. The node [u] is assigned
   * the value of [value] if it is specified. Otherwise it defaults to the
   * value `null`. If [u] is `null` then a unique id is assigned to the node
   * and returned by this function. If a node with the id [u] is already a
   * member of this graph this function throws an `Exception`.
   *
   * After the node has been added, its value can be read and updated using
   * [node].
   */
  Object addNode([u=null, value=null]) {
    if (u == null) {
      do {
        u = "_${++this._nextId}";
      } while (this.hasNode(u));
    } else if (this.hasNode(u)) {
      throw new Exception("Graph already has node '$u'");
    }
    this._nodes[u] = { 'id': u, 'value': value };
    return u;
  }

  /**
   * Removes a node from this graph that has the id [u]. Any edges incident on
   * the node [u] are also removed. If this graph does not contain a node [u]
   * then this function will raise an [Exception].
   */
  delNode(u) {
    this._strictGetNode(u);
    this.incidentEdges(u).forEach((e) { this.delEdge(e); });//, this);
    this._nodes.remove(u);
  }

  // inMap and outMap are opposite sides of an incidence map. For example, for
  // Graph these would both come from the _incidentEdges map, while for Digraph
  // they would come from _inEdges and _outEdges.
  _addEdge(e, u, v, value, Map inMap, Map outMap) {
    this._strictGetNode(u);
    this._strictGetNode(v);

    if (e == null) {
      do {
        e = "_${++this._nextId}";
      } while (this.hasEdge(e));
    }
    else if (this.hasEdge(e)) {
      throw new Exception("Graph already has edge '$e'");
    }

    this._edges[e] = { 'id': e, 'u': u, 'v': v, 'value': value };
    addEdgeToMap(inMap[v], u, e);
    addEdgeToMap(outMap[u], v, e);

    return e;
  }

  // See note for _addEdge regarding inMap and outMap.
  _delEdge(e, Map inMap, Map outMap) {
    var edge = this._strictGetEdge(e);
    delEdgeFromMap(inMap[edge['v']], edge['u'], e);
    delEdgeFromMap(outMap[edge['u']], edge['v'], e);
    this._edges.remove(e);
  }

  /**
   * Creates a new graph that contains all of the nodes and edges from this
   * graph.
   */
  BaseGraph copy() {
    var copy = _like();//new /*this.*/constructor();
    copy.graph(this.graph());
    this.eachNode((u, value) { copy.addNode(u, value); });
    this.eachEdge((e, u, v, value) { copy.addEdge(e, u, v, value); });
    copy._nextId = this._nextId;
    return copy;
  }

  /**
   * Applies the function `f(u)` to each node in this graph and returns
   * a new graph that only includes those nodes for which [filter] returns
   * `true`. Edges that have both incident nodes in the new graph are also
   * copied to the new graph.
   *
   *     filter(u) { return u == 1 || u == 2; }
   *
   *     graph.addNode(1, "node-1");
   *     graph.addNode(2, "node-2");
   *     graph.addNode(3, "node-3");
   *     graph.addEdge(null, 1, 2, "edge-1-2");
   *     graph.addEdge(null, 2, 3, "edge-2-3");
   *
   *     var copy = graph.filterNodes(filter);
   *     copy.nodes();
   *     // => [1, 2]
   *
   *     copy.node(1);
   *     // => "node-1"
   *
   *     copy.neighbors(2);
   *     // => [1]
   */
  BaseGraph filterNodes(bool filter(u)) {
    var copy = _like();//new /*this.*/constructor();
    copy.graph(this.graph());
    this.eachNode((u, value) {
      if (filter(u)) {
        copy.addNode(u, value);
      }
    });
    this.eachEdge((e, u, v, value) {
      if (copy.hasNode(u) && copy.hasNode(v)) {
        copy.addEdge(e, u, v, value);
      }
    });
    return copy;
  }

  _strictGetNode(u) {
    var node = this._nodes[u];
    if (node == null) {
      throw new Exception("Node '$u' is not in graph");
    }
    return node;
  }

  _strictGetEdge(e) {
    var edge = this._edges[e];
    if (edge == null) {
      throw new Exception("Edge '$e' is not in graph");
    }
    return edge;
  }

  /**
   * Returns the ids of all nodes that are adjacent to the node with the id
   * [u]. If [u] is not a member of this graph this function throws an
   * [Exception].
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *     graph.addEdge(null, 1, 2);
   *
   *     graph.neighbors(1);
   *     // => [2]
   *
   *     graph.neighbors(2);
   *     // => [1]
   */
  List neighbors(u);

  /**
   * Returns an arrray of ids for all edges in this graph that are incident
   * on the node [u]. If the node [u] is not in this graph this function
   * raises an [Exception].
   *
   * Optionally the id of another node, [v], may be be specified. This causes
   * the results to be filtered such that only edges between [u] and [v] are
   * included in the returned array. If the node [v] is specified but not a
   * member of this graph this function raises an [Exception].
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *     graph.addNode(3);
   *     graph.addEdge("A", 1, 2);
   *     graph.addEdge("B", 2, 3);
   *     graph.addEdge("C", 3, 1);
   *     graph.addEdge("D", 3, 2);
   *
   *     graph.incidentEdges(3);
   *     // => [ "B", "C", "D" ]
   *
   *     graph.inEdges(3, 2);
   *     // => [ "B", "D" ]
   */
  List incidentEdges(u, [v=null]);

  /**
   * Adds a new edge to this graph with the id [e] from/between a node
   * with the id [u] to/and a node with the id [v]. The edge [e] is
   * assigned the value of [value] if it is specified. Otherwise it defaults
   * to the value `null`.  If [e] is `null` this graph will assign an arbitrary
   * id to the edge. This function returns the id of the edge. It will throw an
   * `Exception` if [u] or [v] are not members of this graph or if [e] is
   * already a member of this graph.
   *
   * After the edge has been added, its value can be read and updated using
   * [edge].
   */
  addEdge(e, u, v, [value=null]);

  /**
   * Removes an edge in this graph with the id [e]. If no edge in this graph
   * has the id [e] this function will thrown an [Exception].
   */
  delEdge(e);

  parent(u, [parent=undefined]) => null;

  List children(u) => [];

  bool isDirected();

  bool isCompound();

  BaseGraph _like();
}

addEdgeToMap(Map map, v, e) {
  (map.containsKey(v) ? map[v] : (map[v] = new Set())).add(e);
}

delEdgeFromMap(Map<Object, Set> map, v, e) {
  var vEntry = map[v];
  vEntry.remove(e);
  if (vEntry.length == 0) {
    map.remove(v);
  }
}

