part of graphlib;
//var Graph = require("./Graph"),
//    compoundify = require("./compoundify");

const undefined = Math.E*Math.PI;

/**
 * Represents a compound
 * [undirected](https://en.wikipedia.org/wiki/Undirected_graph)
 * [multigraph](https://en.wikipedia.org/wiki/Multigraph).
 * It differs from [Graph] in that its nodes can have children.
 */
class CGraph extends Graph {//with Compoundify {

  CGraph();

  factory CGraph.fromGraph(src) {
    var g = new CGraph(),
        graphValue = src.graph();

    if (graphValue != null) {
      g.graph(graphValue);
    }

    src.eachNode((u, value) {
      if (value == null) {
        g.addNode(u);
      } else {
        g.addNode(u, value);
      }
    });
    src.eachEdge((e, u, v, value) {
      if (value == null) {
        g.addEdge(null, u, v);
      } else {
        g.addEdge(null, u, v, value);
      }
    });
    return g;
  }

  /** Always returns `true`. */
  bool isCompound() => true;

  toString() {
    return "CGraph: ${_nodes.length} node(s) ${_edges.length} edge(s)";
  }

  BaseGraph _like() => new CGraph();


  /* TODO Remove duplication in CDigraph. */

  // Map of object id -> parent id (or null for root graph)
  Map _parents = {};

  // Map of id (or null) -> children set
  Map<Object, Set> _children = {null: new Set()};

  /**
   * This function returns the current parent of the node [u] if only one
   * argument is supplied. If two arguments are supplied this function sets
   * the parent of [u] to the node [parent]. If [parent] is `null` this
   * effectively sets the parent to the root graph.
   *
   * This function will throw an [Exception] if either [u] or [parent] are not
   * in this graph.
   *
   *     graph.addNode(1);
   *     graph.parent(1);
   *     // => null
   *
   *     graph.addNode("sg1");
   *     graph.parent(1, "sg1");
   *     graph.parent(1);
   *     // => "sg1"
   */
  parent(u, [parent=undefined]) {
    _strictGetNode(u);

    if (parent == undefined) {
      return _parents[u];
    }

    if (u == parent) {
      throw new Exception("Cannot make " + u + " a parent of itself");
    }
    if (parent != null) {
      _strictGetNode(parent);
    }

    _children[_parents[u]].remove(u);
    _parents[u] = parent;
    _children[parent].add(u);
  }

  /**
   * Returns the ids of all children of the node with the id [u]. The id
   * `null` can be used to get immediate children of the root graph.
   *
   * If [u] is not in this graph this function will throw an [Exception].
   *
   *     graph.children(null);
   *     // => []
   *
   *     graph.addNode(1);
   *     graph.children(null);
   *     // => [ 1 ]
   *
   *     g.addNode("sg1");
   *     g.children(null);
   *     // => [ 1, "sg1" ]
   *
   *     g.parent(1, "sg1");
   *     g.children(null);
   *     // => [ "sg1" ]]
   *     g.children("sg1");
   *     // => [ 1 ]
   */
  List children(u) {
    if (u != null) {
      _strictGetNode(u);
    }
    return _children[u].toList();
  }

  addNode([u=null, value=null]) {
    u = super.addNode(u, value);
    _parents[u] = null;
    _children[u] = new Set();
    _children[null].add(u);
    return u;
  }

  delNode(u) {
    // Promote all children to the parent of the subgraph
    var parent = this.parent(u);
    _children[u].toSet().forEach((child) {
      this.parent(child, parent);
    });//, this);

    _children[parent].remove(u);
    _parents.remove(u);
    _children.remove(u);

    return super.delNode(u);
  }

  copy() {
    var copy = super.copy();
    nodes().forEach((u) {
      copy.parent(u, parent(u));
    });//, this);
    return copy;
  }

  filterNodes(filter) {
    var self = this,
        copy = super.filterNodes(filter);

    var parents = {};
    findParent(u) {
      var parent = self.parent(u);
      if (parent == null || copy.hasNode(parent)) {
        parents[u] = parent;
        return parent;
      } else if (parents.containsKey(parent)) {
        return parents[parent];
      } else {
        return findParent(parent);
      }
    }

    copy.eachNode((u, _) { copy.parent(u, findParent(u)); });

    return copy;
  }

}