library graphlib.converter.json;
/**
 * The `converter` modules provide a way to convert a graph into some other
 * form and back. Currently, we only support a JSON format. The JSON format
 * is reversible, so the following assertion passes:
 *
 *     var decode = require("graphlib").converter.json.decode,
 *         encode = require("graphlib").converter.json.encode;
 *
 *     var encoded = encode(graph);
 *     var decoded = decode(encoded.nodes, encoded.edges, encoded.type);
 *     assert.deepEqual(graph, decoded);
 */

import '../graphlib.dart';

/**
 * Creates a graph of type `Ctor`, or [Digraph] if `Ctor` is `null`, with
 * the given [nodes] and [edges]. `Ctor` may be either a constructor function
 * or one of the strings `"graph"`, `"digraph"`, `"cgraph"`, or `"cdigraph"`.
 *
 * [nodes] must be an array of entries of the form `{ [id], [value],
 * [children] }`. If `id` is not set then a unique id will be assigned to the
 * node. If `value` is not set then the node will be assigned the `null`
 * value. `children` is only used when `Ctor is CGraph || Ctor is CDigraph`
 * and if it is set it must contain an array of node ids in the graph.
 *
 * [edges] must be an array of entries of the form `{ [id], u, v, [value] }`.
 * If `id` is not set then a unique id will be assigned to the edge. `u`
 * represents the source in a directed graph and one of the incident nodes in
 * an undirected graph. `v` represents the target in a directed graph or the
 * other incident node in a directed graph. If `value` is not set then the
 * edge will be assigned the `null` value.
 *
 * Here's an example of decoding a simple directed graph:
 *
 *     var nodes = [{'id': 1}, {'id': 2}];
 *     var edges = [{'id': "A", 'u': 1, 'v': 2}, {'id': "B", 'u: 2, 'v': 1}];
 *     var graph = decode(nodes, edges);
 *
 *     graph.nodes();
 *     // => [1, 2]
 *
 *     graph.edges();
 *     // => ["A", "B"]
 *
 *     graph.source("A");
 *     // => 1
 *
 *     graph.target("A");
 *     // => 2
 *
 * Here's an exmple of decoding a compound directed graph:
 *
 *       var nodes = [{'id': "sg1", 'children': [1, 2]},
 *                    {'id': 1},
 *                    {'id': 2}];
 *       var graph = decode(nodes, [], () => new CGraph());
 *
 *       graph.nodes();
 *       // => ["sg1", 1, 2]
 *
 *       graph.children(null);
 *       // => ["sg1"]
 *
 *       graph.children("sg1");
 *       // => [1, 2]
 */
decode(List nodes, List edges, [Ctor = null]) {
  if (Ctor == null) {
    Ctor = () => new Digraph();
  }

//  if (typeOf(nodes) != "Array") {
//    throw new Error("nodes is not an Array");
//  }
//
//  if (typeOf(edges) != "Array") {
//    throw new Error("edges is not an Array");
//  }

  if (Ctor is String) {
    switch(Ctor) {
      case "graph": Ctor = () => new Graph(); break;
      case "digraph": Ctor = () => new Digraph(); break;
      case "cgraph": Ctor = () => new CGraph(); break;
      case "cdigraph": Ctor = () => new CDigraph(); break;
      default: throw new Exception("Unrecognized graph type: " + Ctor);
    }
  }

  BaseGraph graph = Ctor();

  nodes.forEach((u) {
    graph.addNode(u['id'], u['value']);
  });

  // If the graph is compound, set up children...
  //if (graph.parent) {
  if (graph.isCompound()) {
    nodes.forEach((u) {
      if (u.containsKey('children')) {
        u['children'].forEach((v) {
          graph.parent(v, u['id']);
        });
      }
    });
  }

  edges.forEach((e) {
    graph.addEdge(e['id'], e['u'], e['v'], e['value']);
  });

  return graph;
}

/**
 * This function converts the given [graph] into an object of the form
 * `{ nodes, edges, type }`. These `nodes` and `edges` act the same as the
 * equivalent parameters for the [decode] function. The `type` property will
 * be one of the strings `"graph"`, `"digraph"`, `"cgraph"`, or `"cdigraph"`.
 *
 *     var graph = new Digraph();
 *     graph.addNode(1);
 *     graph.addNode(2, "foo");
 *     graph.addEdge("A", 1, 2, "bar");
 *
 *     encode(graph);
 *     // => { nodes: [{ 'id': 1, 'value': null },
 *     //              { 'id': 2, 'value': "foo" }],
 *     //      edges: [{ 'id': "A", 'u': 1, 'v': 2, 'value': "bar" }],
 *     //      type: "digraph" }
 */
Map encode(BaseGraph graph) {
  var nodes = [];
  var edges = [];

  graph.eachNode((u, value) {
    var node = {'id': u, 'value': value};
    var children;
    //if (graph.children) {
    if (graph is CGraph || graph is CDigraph) {
      if (graph is CGraph) children = graph.children(u);
      if (graph is CDigraph) children = graph.children(u);
      if (children.length != 0) {
        node['children'] = children;
      }
    }
    nodes.add(node);
  });

  graph.eachEdge((e, u, v, value) {
    edges.add({'id': e, 'u': u, 'v': v, 'value': value});
  });

  var type;
  if (graph is CDigraph) {
    type = "cdigraph";
  } else if (graph is CGraph) {
    type = "cgraph";
  } else if (graph is Digraph) {
    type = "digraph";
  } else if (graph is Graph) {
    type = "graph";
  } else {
    throw new Exception("Couldn't determine type of graph: $graph");
  }

  return { 'nodes': nodes, 'edges': edges, 'type': type };
}

//typeOf(obj) {
//  return Object.prototype.toString.call(obj).slice(8, -1);
//}
