part of graphlib;
/*
 * This file is organized with in the following order:
 *
 * Exports
 * Graph constructors
 * Graph queries (e.g. nodes(), edges()
 * Graph mutators
 * Helper functions
 */

/**
 * Represents a [directed](https://en.wikipedia.org/wiki/Directed_graph)
 * [multigraph](https://en.wikipedia.org/wiki/Multigraph).
 */
class Digraph extends BaseGraph {

  /** Map of sourceId -> {targetId -> Set of edge ids} */
  Map<Object, Map<Object, Set>> _inEdges = {};

  /** Map of targetId -> {sourceId -> Set of edge ids} */
  Map<Object, Map<Object, Set>> _outEdges = {};

  /** Always returns `true`. */
  isDirected() => true;

  /** Always returns `false`. */
  bool isCompound() => false;

  /**
   * Returns the ids of all nodes that are successors of the node with the id
   * [u]. That is, all nodes that have the node [u] as their source are
   * returned. If [u] is not a member of this graph this function throws an
   * [Exception].
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *     graph.addEdge(null, 1, 2);
   *
   *     graph.successors(1);
   *     // => [2]
   *
   *     graph.successors(2);
   *     // => []
   */
  List successors(u) {
    this._strictGetNode(u);
    return this._outEdges[u].keys
        .map((v) { return this._nodes[v]['id']; }).toList();//, this);
  }

  /**
   * Returns the ids of all nodes that are predecessors of the node with the
   * id [u]. That is, all nodes that have the node [u] as their target are
   * returned. If `u` is not a member of this graph this function throws an
   * [Exception].
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *     graph.addEdge(null, 1, 2);
   *
   *     graph.predecessors(1);
   *     // => []
   *
   *     graph.predecessors(2);
   *     // => [1]
   */
  List predecessors(u) {
    this._strictGetNode(u);
    return this._inEdges[u].keys
        .map((v) { return this._nodes[v]['id']; }).toList();//, this);
  }

  List neighbors(u) {
    //return Set.union([this.successors(u), this.predecessors(u)]).keys();
    return util.union([this.successors(u), this.predecessors(u)]).toList();
  }

  /**
   * Returns the ids of all nodes that are in this graph that are sources. A
   * source in a directed graph is a node that has no in-edges.
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *     graph.addEdge(null, 1, 2);
   *
   *     graph.sources();
   *     // => [1]
   */
  List sources() {
    var self = this;
    return this._filterNodes((u) {
      // This could have better space characteristics if we had an inDegree
      // function.
      return self.inEdges(u).length == 0;
    });
  }

  /**
   * Returns the ids of all nodes that are in this graph that are sinks. A
   * sink in a directed graph is a node that has no out-edges.
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *     graph.addEdge(null, 1, 2);
   *
   *     graph.sinks();
   *     // => [2]
   */
  List sinks() {
    var self = this;
    return this._filterNodes((u) {
      // This could have better space characteristics if we have an outDegree
      // function.
      return self.outEdges(u).length == 0;
    });
  }

  /**
   * Returns the source node incident on the edge with the id [e]. If no such
   * edge exists in this graph this function throws an [Exception].
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *     graph.addEdge("A", 1, 2);
   *     graph.source("A");
   *     // => 1
   */
  source(e) => this._strictGetEdge(e)['u'];

  /**
   * Returns the target node incident on the edge with the id [e]. If no such
   * edge exists in this graph this function throws an [Exception].
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *     graph.addEdge("A", 1, 2);
   *     graph.target("A");
   *     // => 2
   */
  target(e) => this._strictGetEdge(e)['v'];

  /**
   * Returns an array of ids for all edges in the graph that have the node
   * [target] as their target. If the node [target] is not in the graph this
   * function raises an [Exception].
   *
   * Optionally a [source] node can also be specified. This causes the results
   * to be filtered such that only edges from [source] to [target] are
   * included. If the node [source] is specified but is not in the graph then
   * this function raises an [Exception].
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *     graph.addNode(3);
   *     graph.addEdge("A", 1, 2);
   *     graph.addEdge("B", 2, 3);
   *     graph.addEdge("C", 3, 1);
   *     graph.addEdge("D", 3, 2);
   *
   *     graph.inEdges(2);
   *     // => [ "A", "D" ]
   *
   *     graph.inEdges(2, 3);
   *     // => [ "D" ]
   *
   * Note that `graph.inEdges(target, source)` yields the same result as
   * `graph.outEdges(source, target)`.
   */
  List inEdges(target, [source=null]) {
    this._strictGetNode(target);
    //var results = Set.union(util.values(this._inEdges[target])).keys();
    var results = util.union(this._inEdges[target].values).toList();
    if (source != null) {
      this._strictGetNode(source);
      results = results.where((e) { return this.source(e) == source; }).toList();//, this);
    }
    return results;
  }

  /**
   * Returns an array of ids for all edges in the graph that have the node
   * [source] as their source. If the node [source] is not in the graph this
   * function raises an [Exception].
   *
   * Optionally a [target] node may also be specified. This causes the results
   * to be filtered such that only edges from [source] to [target] are
   * included. If the node [target] is specified but is not in the graph then
   * this function raises an [Exception].
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *     graph.addNode(3);
   *     graph.addEdge("A", 1, 2);
   *     graph.addEdge("B", 2, 3);
   *     graph.addEdge("C", 3, 1);
   *     graph.addEdge("D", 3, 2);
   *
   *     graph.outEdges(3);
   *     // => [ "C", "D" ]
   *
   *     graph.inEdges(3, 2);
   *     // => [ "D" ]
   */
  List outEdges(source, [target=null]) {
    this._strictGetNode(source);
    //var results = Set.union(util.values(this._outEdges[source])).keys();
    var results = util.union(this._outEdges[source].values).toList();
    if (target != null) {
      this._strictGetNode(target);
      results = results.where((e) { return this.target(e) == target; }).toList();//, this);
    }
    return results;
  }

  List incidentEdges(u, [v=null]) {
    if (v != null) {
      return util.union([this.outEdges(u, v), this.outEdges(v, u)]).toList();
    } else {
      return util.union([this.inEdges(u), this.outEdges(u)]).toList();
    }
  }

  /** Returns a string representation of this graph. */
  toString() {
    return "Digraph: ${_nodes.length} node(s) ${_edges.length} edge(s)";
  }

  addNode([u=null, value=null]) {
    //u = BaseGraph.prototype.addNode.call(this, u, value);
    u = super.addNode(u, value);
    this._inEdges[u] = {};
    this._outEdges[u] = {};
    return u;
  }

  delNode(u) {
    //BaseGraph.prototype.delNode.call(this, u);
    super.delNode(u);
    this._inEdges.remove(u);
    this._outEdges.remove(u);
  }

  /**
   * Adds a new edge to the graph with the id [e] from a node with the id
   * [source] to a node with an id [target] and assigns it the value [value].
   * This graph allows more than one edge from [source] to `target` as long
   * as the id [e] is unique in the set of edges. If [e] is `null` the graph
   * will assign a unique identifier to the edge.
   *
   * If [source] or [target] are not present in the graph this function will
   * throw an [Exception].
   */
  addEdge(e, source, target, [value=null]) {
    return super._addEdge.call(e, source, target, value,
        this._inEdges, this._outEdges);
  }

  delEdge(e) {
    super._delEdge(e, this._inEdges, this._outEdges);
  }

  // Unlike BaseGraph.filterNodes, this helper just returns nodes that
  // satisfy a predicate.
  _filterNodes(pred) {
    var filtered = [];
    this.eachNode((u, _) {
      if (pred(u)) {
        filtered.add(u);
      }
    });
    return filtered;
  }

  /**
   * Returns a new undirected graph using the nodes and edges from this graph.
   * The new graph will have the same nodes, but the edges will be made
   * undirected. Edge ids are preserved in this transformation.
   *
   *     digraph.addNode(1);
   *     digraph.addNode(2);
   *     digraph.addNode(3);
   *     digraph.addEdge("A", 1, 2);
   *     digraph.addEdge("B", 2, 3);
   *
   *     var graph = digraph.toGraph();
   *     graph is Graph;
   *     // => true
   *
   *     graph.nodes();
   *     // => [ 1, 2, 3 ]
   *
   *     graph.neighbors(2);
   *     // => [ 1, 3 ]
   *
   *     graph.edges();
   *     // => [ "A", "B" ]
   */
  Graph toGraph() => asUndirected();

  /** Synonym for [toGraph]. */
  asUndirected() {
    var g = new Graph();
    this.eachNode((u, value) { g.addNode(u, value); });
    this.eachEdge((e, u, v, value) {
      g.addEdge(e, u, v, value);
    });
    return g;
  }

  BaseGraph _like() => new Digraph();
}
