part of graphlib;
/*
 * This file is organized with in the following order:
 *
 * Exports
 * Graph constructors
 * Graph queries (e.g. nodes(), edges()
 * Graph mutators
 * Helper functions
 */

/**
 * Represents an [undirected](https://en.wikipedia.org/wiki/Undirected_graph)
 * [multigraph](https://en.wikipedia.org/wiki/Multigraph).
 */
class Graph extends BaseGraph {

  /** Map of nodeId -> { otherNodeId -> Set of edge ids } */
  Map<Object, Map<Object, Set>> _incidentEdges = {};

  /** Always returns `false`. */
  bool isDirected() => false;

  /** Always returns `false`. */
  bool isCompound() => false;

  /**
   * Returns all nodes that are adjacent to the node with the id [u].
   * If [u] is not a member of this graph this function throws an [Exception].
   */
  List neighbors(u) {
    this._strictGetNode(u);
    return this._incidentEdges[u].keys
                 .map((v) { return this._nodes[v]['id']; }).toList();//, this);
  }

  List incidentEdges(u, [v=null]) {
    this._strictGetNode(u);
    if (v != null) {
      this._strictGetNode(v);
      //return this._incidentEdges[u].containsKey(v) ? this._incidentEdges[u][v].keys : [];
      return this._incidentEdges[u].containsKey(v) ? this._incidentEdges[u][v].toList() : [];
    } else {
      //return Set.union(util.values(this._incidentEdges[u])).keys;
      return util.union(this._incidentEdges[u].values).toList();
    }
  }

  /** Returns a string representation of this graph. */
  toString() {
    //return "Graph " + JSON.encode(this);//, null, 2);
    return "Graph: ${_nodes.length} node(s) ${_edges.length} edge(s)";
  }

  addNode([u=null, value=null]) {
    //u = BaseaddNode.call(this, u, value);
    u = super.addNode(u, value);
    this._incidentEdges[u] = {};
    return u;
  }

  delNode(u) {
    //BasedelNode.call(this, u);
    super.delNode(u);
    this._incidentEdges.remove(u);
  }

  addEdge(e, u, v, [value=null]) {
    return super._addEdge(e, u, v, value,
        this._incidentEdges, this._incidentEdges);
  }

  delEdge(e) {
    super._delEdge(e, this._incidentEdges, this._incidentEdges);
  }

  /**
   * Returns a new directed graph using the nodes and edges from this graph.
   * The new graph will have the same nodes, but will have twice the number
   * of edges: each edge is split into two edges with opposite directions.
   * Edge ids, consequently, are not preserved by this transformation.
   *
   *     graph.addNode(1);
   *     graph.addNode(2);
   *     graph.addNode(3);
   *     graph.addEdge(null, 1, 2);
   *     graph.addEdge(null, 2, 3);
   *     graph.size();
   *     // => 3
   *
   *     var digraph = graph.toDigraph();
   *     digraph is Digraph;
   *     // => true
   *
   *     digraph.nodes();
   *     // => [ 1, 2, 3 ]
   *
   *     digraph.size();
   *     // => 6 /* twice the number of edges */
   *
   *     digraph.successors(2);
   *     // => [ 1, 3 ]
   *
   *     digraph.edges();
   *     // => [ "_ANON-1", "_ANON-2", "_ANON-3", "_ANON-4" ]
   */
  Digraph toDigraph() => asDirected();

  /** Synonym for [toDigraph]. */
  Digraph asDirected() {
    var g = new Digraph();
    this.eachNode((u, value) { g.addNode(u, value); });
    this.eachEdge((e, u, v, value) {
      g.addEdge(null, u, v, value);
      g.addEdge(null, v, u, value);
    });
    return g;
  }

  BaseGraph _like() => new Graph();
}