part of graphlib.test;

abstractCompoundifyTest(name, BaseGraph Constructor(), superName, BaseGraph SuperConstructor()) {
  var g;

  group('compound', () {
    setUp(() {
      g = Constructor();
    });

    group('new ' + name + '()', () {
      test('has no nodes', () {
        expect(g.order(), equals(0));
        expect(g.nodes().length, equals(0));
      });

      test('has no edges', () {
        expect(g.size(), equals(0));
        expect(g.edges().length, equals(0));
      });
    });

    group('parent', () {
      test('throws if the object is not in the graph', () {
        expect(() { g.parent('node-1'); }, throws);
      });

      test('defaults to null (root) for new nodes', () {
        g.addNode(1);
        expect(g.parent(1), isNull);
      });

      test('throws an error for edges', () {
        g.addNode(1);
        g.addEdge('A', 1, 1);
        expect(() { g.parent('A'); }, throws);
      });

      test('throws an error for a null or undefined parameter', () {
        expect(() { g.parent(); }, throws);
        expect(() { g.parent(null); }, throws);
      });

      test('sets the parent with 2 arguments', () {
        g.addNode(1);
        g.addNode('sg1');
        g.parent(1, 'sg1');
        expect(g.parent(1), equals('sg1'));
        expect(g.children('sg1').length, equals(1));
      });

      test('removes the current parent with 2 arguments', () {
        g.addNode(1);
        g.addNode('sg1');
        g.parent(1, 'sg1');
        expect(g.children(null), equals(['sg1']));
      });

      test('sets the parent to root if it is null', () {
        g.addNode(1);
        g.addNode('sg1');
        g.parent(1, 'sg1');
        g.parent(1, null);
        expect(g.parent(1), equals(null));
        expect(g.children('sg1').length, equals(0));
        expect(g.children(null).length, equals(2));
      });

      test('does not allow a node to be its own parent', () {
        g.addNode('sg1');
        expect(() { g.parent('sg1', 'sg1'); }, throws);
      });
    });

    group('children', () {
      test('returns an empty list for an empty subgraph', () {
        g.addNode('sg1');
        expect(g.children('sg1'), equals([]));
      });

      test('returns a list of all nodes in the root graph with the null parameter', () {
        g.addNode(1);
        g.addEdge(2, 1, 1);
        expect(g.children(null), equals([1]));
      });
    });

    group('delNode', () {
      test('promotes children to the root when the subgraph was off the root', () {
        g.addNode('sg1');
        g.addNode(1);
        g.parent(1, 'sg1');
        g.delNode('sg1');
        expect(g.parent(1), equals(null));
      });

      test('promotes children to the parent of the subgraph', () {
        g.addNode('sg1');
        g.addNode('sg2');
        g.addNode(1);
        g.parent('sg1', 'sg2');
        g.parent(1, 'sg1');
        g.delNode('sg1');
        expect(g.parent(1), equals('sg2'));
      });

      test('removes the parent information', () {
        g.addNode('sg1');
        g.delNode('sg1');
        expect(() { g.parent('sg1'); }, throws);
      });

      test('removes the children information', () {
        g.addNode('sg1');
        g.delNode('sg1');
        expect(() { g.children('sg1'); }, throws);
      });

      test('removes the node from its parent', () {
        g.addNode(1);
        g.delNode(1);
        expect(g.children(null).length, equals(0));
      });

      test('can remove a node created with a automatically assigned id', () {
        var id = g.addNode();
        g.delNode(id);
        expect(g.nodes().length, equals(0));
      });
    });

    group('filterNodes', () {
      test('includes subgraphs', () {
        g.addNode('sg1');
        g.addNode(1);
        g.parent(1, 'sg1');
        var g2 = g.filterNodes(all());
        expect(g2.parent(1), equals('sg1'));
        expect(g2.parent('sg1'), equals(null));
      });

      test('includes nested subgraphs', () {
        g.addNode('sg1');
        g.addNode('sg2');
        g.parent('sg2', 'sg1');
        g.addNode(1);
        g.parent(1, 'sg2');
        g.addNode(2);
        g.parent(2, 'sg2');
        var g2 = g.filterNodes(all());
        expect(g2.parent(1), equals('sg2'));
        expect(g2.parent(2), equals('sg2'));
        expect(g2.parent('sg2'), equals('sg1'));
        expect(g2.parent('sg1'), equals(null));
      });

      test('handles promotion if a parent subgraph node is not included', () {
        g.addNode('sg2');
        g.addNode('sg1');
        g.parent('sg1', 'sg2');
        g.addNode(1);
        g.parent(1, 'sg1');
        g.addNode(2);
        g.parent(2, 'sg1');
        var g2 = g.filterNodes(nodesFromList(['sg2', 1, 2]));
        expect(g2.parent(1), equals('sg2'));
        expect(g2.parent(2), equals('sg2'));
        expect(g2.parent('sg2'), equals(null));
      });
    });

    /*group('from' + superName, () {
      var fromSuper = Constructor['from' + superName];

      test('returns a ' + name, () {
        var g = new SuperConstructor();
        expect(fromSuper(g), instanceOf(Constructor));
      });

      test('includes the graph value', () {
        var g = new SuperConstructor();
        g.graph({'a': 'a-value'});
        expect(fromSuper(g).graph(), equals({'a': 'a-value'}));
      });

      test('includes nodes from the source graph', () {
        var g = new SuperConstructor();
        g.addNode(1);
        g.addNode(2);
        expect(fromSuper(g).nodes(), same([1, 2]));
      });

      test('includes node attributes from the source graph', () {
        var g = new SuperConstructor();
        g.addNode(1, {'a': 'a-value'});
        expect(fromSuper(g).node(1), equals({'a': 'a-value'}));
      });

      test('includes edges from the source graph', () {
        var g = new SuperConstructor();
        g.addNode(1);
        var edgeId = g.addEdge(null, 1, 1);
        expect(fromSuper(g).edges(), same([edgeId]));
      });

      test('includes edge attributes from the source graph', () {
        var g = new SuperConstructor();
        g.addNode(1);
        var edgeId = g.addEdge(null, 1, 1, {'a': 'a-value'});
        expect(fromSuper(g).edge(edgeId), equals({'a': 'a-value'}));
      });

      test('has the same incidentNodes for edges from the source graph', () {
        var g = new SuperConstructor();
        g.addNode(1);
        g.addNode(2);
        var edgeId = g.addEdge(null, 1, 2);
        expect(fromSuper(g).incidentNodes(edgeId), equals([1, 2]));
      });
    });*/
  });
}
