part of graphlib.test;

abstractDigraphTest(Digraph makeDigraph()) {
  abstractBaseGraphTest(makeDigraph);

  Digraph g;

  setUp(() {
    g = makeDigraph();
  });

  group('asUndirected', () {
    Digraph g;

    setUp(() {
      g = new Digraph();
      g.addNode(1);
      g.addNode(2);
      g.addNode(3);
      g.addNode(4);
      g.addEdge('A', 1, 2, 'A');
      g.addEdge('B', 2, 3, 'B');
      g.addEdge('C', 3, 4, 'C');
      g.addEdge('D', 4, 1, 'D');
      g.addEdge('E', 4, 2, 'E');
    });

    test('returns an undirected graph', () {
      expect(g.asUndirected().isDirected(), isFalse);
    });

    test('has the same nodes', () {
      expect(g.asUndirected().nodes()..sort(), equals([1, 2, 3, 4]));
    });

    test('has the same size', () {
      expect(g.asUndirected().size(), equals(5));
    });

    test('preserves edge ids', () {
      var ug = g.asUndirected();
      expect(ug.incidentNodes('A')..sort(), equals([1, 2]));
      expect(ug.incidentNodes('B')..sort(), equals([2, 3]));
      expect(ug.incidentNodes('C')..sort(), equals([3, 4]));
      expect(ug.incidentNodes('D')..sort(), equals([1, 4]));
      expect(ug.incidentNodes('E')..sort(), equals([2, 4]));
    });

    test('preserves edge labels', () {
      var ug = g.asUndirected();
      expect(ug.edge(ug.incidentEdges(1, 2)[0]), equals('A'));
      expect(ug.edge(ug.incidentEdges(2, 3)[0]), equals('B'));
      expect(ug.edge(ug.incidentEdges(3, 4)[0]), equals('C'));
      expect(ug.edge(ug.incidentEdges(4, 1)[0]), equals('D'));
      expect(ug.edge(ug.incidentEdges(4, 2)[0]), equals('E'));
    });
  });

  group('isDirected', () {
    test('always returns true', () {
      expect(new Digraph().isDirected(), isTrue);
    });
  });

  group('sources', () {
    test('returns all nodes for a graph with no edges', () {
      var g = new Digraph();
      g.addNode('a');
      g.addNode('b');
      g.addNode('c');
      expect(g.sources()..sort(), equals(['a', 'b', 'c']));
    });

    test('returns only nodes that have no in-edges', () {
      var g = new Digraph();
      g.addNode('a');
      g.addNode('b');
      g.addNode('c');
      g.addEdge(null, 'a', 'b');
      expect(g.sources()..sort(), equals(['a', 'c']));
    });
  });

  group('sinks', () {
    test('returns all nodes for a graph with no edges', () {
      var g = new Digraph();
      g.addNode('a');
      g.addNode('b');
      g.addNode('c');
      expect(g.sinks()..sort(), equals(['a', 'b', 'c']));
    });

    test('returns only nodes that have no out-edges', () {
      var g = new Digraph();
      g.addNode('a');
      g.addNode('b');
      g.addNode('c');
      g.addEdge(null, 'a', 'b');
      expect(g.sinks()..sort(), equals(['b', 'c']));
    });
  });

  group('outEdges', () {
    test('returns all out edges from a source', () {
      g.addNode(1);
      g.addNode(2);
      g.addEdge('A', 1, 2);
      g.addEdge('B', 1, 2);
      g.addEdge('C', 1, 1);
      g.addEdge('D', 2, 1);
      g.addEdge('E', 2, 2);

      expect(g.outEdges(1)..sort(), equals(['A', 'B', 'C']));
      expect(g.outEdges(2)..sort(), equals(['D', 'E']));
    });

    test('optionally returns all out edges from a source filtered by target', () {
      g.addNode(1);
      g.addNode(2);
      g.addEdge('A', 1, 2);
      g.addEdge('B', 1, 2);
      g.addEdge('C', 1, 1);
      g.addEdge('D', 2, 1);
      g.addEdge('E', 2, 2);

      expect(g.outEdges(1, 1)..sort(), equals(['C']));
      expect(g.outEdges(1, 2)..sort(), equals(['A', 'B']));
      expect(g.outEdges(2, 1)..sort(), equals(['D']));
      expect(g.outEdges(2, 2)..sort(), equals(['E']));
    });
  });

  group('inEdges', () {
    test('returns all in edges to a target', () {
      g.addNode(1);
      g.addNode(2);
      g.addEdge('A', 1, 2);
      g.addEdge('B', 1, 2);
      g.addEdge('C', 1, 1);
      g.addEdge('D', 2, 1);
      g.addEdge('E', 2, 2);

      expect(g.inEdges(1)..sort(), equals(['C', 'D']));
      expect(g.inEdges(2)..sort(), equals(['A', 'B', 'E']));
    });

    test('optionally returns all in edges to a target filtered by source', () {
      g.addNode(1);
      g.addNode(2);
      g.addEdge('A', 1, 2);
      g.addEdge('B', 1, 2);
      g.addEdge('C', 1, 1);
      g.addEdge('D', 2, 1);
      g.addEdge('E', 2, 2);

      expect(g.inEdges(1, 1)..sort(), equals(['C']));
      expect(g.inEdges(1, 2)..sort(), equals(['D']));
      expect(g.inEdges(2, 1)..sort(), equals(['A', 'B']));
      expect(g.inEdges(2, 2)..sort(), equals(['E']));
    });
  });

  group('addNode', () {
    test('adds a new node to the graph', () {
      g.addNode(1);

      expect(g.nodes(), equals([1]));

      expect(g.successors(1).length, equals(0));
      expect(g.predecessors(1).length, equals(0));
      expect(g.neighbors(1).length, equals(0));
    });
  });

  group('delNode', () {
    test('removes out edges', () {
      g.addNode(1);
      g.addNode(2);
      g.addEdge('1 -> 2', 1, 2);
      g.delNode(1);
      expect(g.predecessors(2).length, equals(0));
    });

    test('removes in edges', () {
      g.addNode(1);
      g.addNode(2);
      g.addEdge('2 -> 1', 2, 1);
      g.delNode(1);
      expect(g.successors(2).length, equals(0));
    });
  });

  group('addEdge', () {
    test('adds a new edge to the graph', () {
      g.addNode(1);
      g.addNode(2);
      g.addEdge('a', 1, 2);

      expect(g.edges(), equals(['a']));

      expect(g.successors(1), equals([2]));
      expect(g.predecessors(2), equals([1]));
      expect(g.neighbors(1), equals([2]));
      expect(g.neighbors(2), equals([1]));
    });
  });

  group('delEdge', () {
    test('removes the specified edge from the graph', () {
      g.addNode(1);
      g.addNode(2);
      g.addEdge('a', 1, 2);
      g.delEdge('a');

      expect(g.edges().length, equals(0));

      expect(g.successors(1).length, equals(0));
      expect(g.predecessors(1).length, equals(0));
      expect(g.neighbors(1).length, equals(0));
    });
  });
}
