import 'package:unittest/unittest.dart';
import 'package:graphlib/graphlib.dart';

//var assert = require('../assert'),
//    decode = require('../..').converter.json.decode,
//    encode = require('../..').converter.json.encode,
//    Digraph = require('../..').Digraph,
//    Graph = require('../..').Graph,
//    CDigraph = require('../..').CDigraph,
//    CGraph = require('../..').CGraph;

main() {
  group('converter.json', () {
    group('decode', () {
      test('defaults to Digraph type', () {
        expect(decode([], []), new isInstanceOf<Digraph>());
      });

      test('can create Graph type', () {
        expect(decode([], [], () => new Graph()), new isInstanceOf<Graph>());
      });

      test('can populate a graph with nodes', () {
        var nodes = [{'id': 1}, {'id': 2}];
        var graph = decode(nodes, []);
        expect(graph.nodes(), unorderedEquals([1, 2]));
        expect(graph.node(1), isNull);
        expect(graph.node(2), isNull);
      });

      test('can populate a graph with node values', () {
        var nodes = [{'id': 1, 'value': 'A'}, {'id': 2, 'value': 'B' }];
        var graph = decode(nodes, []);
        expect(graph.nodes(), unorderedEquals([1, 2]));
        expect(graph.node(1), equals('A'));
        expect(graph.node(2), equals('B'));
      });

      test('can create a graph with auto-assigned node ids', () {
        var nodes = [{'value': 'A'}, {'value': 'B'}];
        var graph = decode(nodes, []);
        expect(graph.order(), equals(2));
        expect(graph.nodes().map((x) { return graph.node(x); }), unorderedEquals(['A', 'B']));
      });

      test('can add edges to the graph', () {
        var nodes = [{'id': 1}, {'id': 2}];
        var edges = [{'id': 'A', 'u': 1, 'v': 2}, {'id': 'B', 'u': 2, 'v': 1}];
        var graph = decode(nodes, edges);
        expect(graph.edges(), unorderedEquals(['A', 'B']));
        expect(graph.source('A'), equals(1));
        expect(graph.target('A'), equals(2));
        expect(graph.source('B'), equals(2));
        expect(graph.target('B'), equals(1));
        expect(graph.edge('A'), isNull);
        expect(graph.edge('B'), isNull);
      });

      test('can add edge values to the graph', () {
        var nodes = [{'id': 1}, {'id': 2}];
        var edges = [{'id': 'A', 'u': 1, 'v': 2, 'value': 'foo'}];
        var graph = decode(nodes, edges);
        expect(graph.edges(), unorderedEquals(['A']));
        expect(graph.edge('A'), equals('foo'));
      });

      test('can create a graph with auto-assigned edge ids', () {
        var nodes = [{'id': 1}, {'id': 2}];
        var edges = [{'u': 1, 'v': 2, 'value': 'foo'}];
        var graph = decode(nodes, edges);
        expect(graph.size(), equals(1));

        var e = graph.edges()[0];
        expect(graph.source(e), equals(1));
        expect(graph.target(e), equals(2));
        expect(graph.edge(e), equals('foo'));
      });

      test('can create a directed graph with compound nodes', () {
        var nodes = [{'id': 'sg1', 'children': [1, 2]},
                     {'id': 1},
                     {'id': 2}];
        var graph = decode(nodes, [], () => new CDigraph());
        expect(graph.nodes(), unorderedEquals(['sg1', 1, 2]));
        expect(graph.children(null), unorderedEquals(['sg1']));
        expect(graph.children('sg1'), unorderedEquals([1, 2]));
      });

      test('can create an undirected graph with compound nodes', () {
        var nodes = [{'id': 'sg1', 'children': [1, 2]},
                     {'id': 1},
                     {'id': 2}];
        var graph = decode(nodes, [], () => new CGraph());
        expect(graph.nodes(), unorderedEquals(['sg1', 1, 2]));
        expect(graph.children(null), unorderedEquals(['sg1']));
        expect(graph.children('sg1'), unorderedEquals([1, 2]));
      });

      test('ignores compound information for non-comound graphs', () {
        var nodes = [{'id': 'sg1', 'children': [1, 2]},
                     {'id': 1},
                     {'id': 2}];
        var graph = decode(nodes, []);
        expect(graph.nodes(), unorderedEquals(['sg1', 1, 2]));
      });

//      test('throws an Error if the nodes argument is not an array', () {
//        expect(() { decode('foo', []); }, throws);
//      });
//
//      test('throws an Error if the edges argument is not an array', () {
//        expect(() { decode([], 'foo'); }, throws);
//      });
    });

    group('encode', () {
      test('can encode an empty Graph', () {
        var graph = new Graph();
        expect(encode(graph), equals({ 'nodes': [], 'edges': [], 'type': 'graph' }));
      });

      test('can encode an empty Digraph', () {
        var graph = new Digraph();
        expect(encode(graph), equals({ 'nodes': [], 'edges': [], 'type': 'digraph' }));
      });

      test('can encode an empty CGraph', () {
        var graph = new CGraph();
        expect(encode(graph), equals({ 'nodes': [], 'edges': [], 'type': 'cgraph' }));
      });

      test('can encode an empty CDigraph', () {
        var graph = new CDigraph();
        expect(encode(graph), equals({ 'nodes': [], 'edges': [], 'type': 'cdigraph' }));
      });

      test('can encode a graph with nodes', () {
        var graph = new Digraph();
        graph.addNode(1);
        graph.addNode(2);

        var encoded = encode(graph);
        expect(encoded['nodes'].map((x) { return x['id']; }), unorderedEquals([1, 2]));
        expect(encoded['edges'], equals([]));
        expect(encoded['type'], equals('digraph'));
      });

      test('can encode a graph with node values', () {
        var graph = new Digraph();
        graph.addNode(1, 'test');
        graph.addNode(2, {'complex': true});

        var encoded = encode(graph);
        expect(encoded['nodes']..sort(idCmp), equals([{'id': 1, 'value': 'test'}, {'id': 2, 'value': {'complex': true}}]));
        expect(encoded['edges'], equals([]));
        expect(encoded['type'], equals('digraph'));
      });

      test('can encode a graph with edges', () {
        var graph = new Digraph();
        graph.addNode(1);
        graph.addNode(2);
        graph.addEdge('A', 1, 2);
        graph.addEdge('B', 2, 1);

        var encoded = encode(graph);
        expect(encoded['nodes']..sort(idCmp), equals([{'id': 1, 'value': null}, {'id': 2, 'value': null}]));
        expect(encoded['edges']..sort(idCmp), equals([{'id': 'A', 'u': 1, 'v': 2, 'value': null},
                                                     {'id': 'B', 'u': 2, 'v': 1, 'value': null}]));
        expect(encoded['type'], equals('digraph'));

      });

      test('can encode a graph with edge values', () {
        var graph = new Digraph();
        graph.addNode(1);
        graph.addNode(2);
        graph.addEdge('A', 1, 2, 'test');
        graph.addEdge('B', 2, 1, {'complex': true});

        var encoded = encode(graph);
        expect(encoded['nodes']..sort(idCmp), equals([{'id': 1, 'value': null}, {'id': 2, 'value': null}]));
        expect(encoded['edges']..sort(idCmp), equals([{'id': 'A', 'u': 1, 'v': 2, 'value': 'test'},
                                                     {'id': 'B', 'u': 2, 'v': 1, 'value': {'complex': true}}]));
        expect(encoded['type'], equals('digraph'));

      });

      test('can encode a compound directed graph', () {
        var graph = new CDigraph();
        graph.addNode('sg1');
        graph.addNode(1);
        graph.parent(1, 'sg1');
        graph.addNode(2);
        graph.parent(2, 'sg1');

        var encoded = encode(graph);
        expect(encoded['nodes']..sort(idCmp), equals([{'id': 1, 'value': null},
                                                     {'id': 2, 'value': null},
                                                     {'id': 'sg1', 'value': null, 'children': [1, 2]}]));
        expect(encoded['edges'], equals([]));
        expect(encoded['type'], equals('cdigraph'));
      });

      test('can encode a compound undirected graph', () {
        var graph = new CGraph();
        graph.addNode('sg1');
        graph.addNode(1);
        graph.parent(1, 'sg1');
        graph.addNode(2);
        graph.parent(2, 'sg1');

        var encoded = encode(graph);
        expect(encoded['nodes']..sort(idCmp), equals([{'id': 1, 'value': null},
                                                     {'id': 2, 'value': null},
                                                     {'id': 'sg1', 'value': null, 'children': [1, 2]}]));
        expect(encoded['edges'], equals([]));
        expect(encoded['type'], equals('cgraph'));
      });
    });

    /*group('encode-decode', () {
      test('Digraph', () {
        var graph = new Digraph();
        graph.addNode(1);
        graph.addNode(2, {'value': 'node 2'});
        graph.addEdge('A', 1, 2, 'forward edge');
        graph.addEdge('B', 2, 1, 'reverse edge');

        var encoded = encode(graph);
        var decoded = decode(encoded['nodes'], encoded['edges'], encoded['type']);
        expect(decoded, new isInstanceOf<Digraph>());
        expect(graph, equals(decoded));
      });

      test('Graph', () {
        var graph = new Graph();
        graph.addNode(1);
        graph.addNode(2, {'value': 'node 2'});
        graph.addEdge('A', 1, 2, 'forward edge');
        graph.addEdge('B', 2, 1, 'reverse edge');

        var encoded = encode(graph);
        var decoded = decode(encoded['nodes'], encoded['edges'], encoded['type']);
        expect(decoded, new isInstanceOf<Graph>());
        expect(graph, equals(decoded));
      });

      test('CDigraph', () {
        var graph = new CDigraph();
        graph.addNode(1);
        graph.addNode(2, {'value': 'node 2'});
        graph.addNode('sg1', 'foo');
        graph.parent(1, 'sg1');
        graph.parent(2, 'sg1');
        graph.addEdge('A', 1, 2, 'forward edge');
        graph.addEdge('B', 2, 1, 'reverse edge');

        var encoded = encode(graph);
        var decoded = decode(encoded['nodes'], encoded['edges'], encoded['type']);
        expect(decoded, new isInstanceOf<CDigraph>());
        expect(graph, equals(decoded));
      });

      test('CGraph', () {
        var graph = new CGraph();
        graph.addNode(1);
        graph.addNode(2, {'value': 'node 2'});
        graph.addNode('sg1', 'foo');
        graph.parent(1, 'sg1');
        graph.parent(2, 'sg1');
        graph.addEdge('A', 1, 2, 'forward edge');
        graph.addEdge('B', 2, 1, 'reverse edge');

        var encoded = encode(graph);
        var decoded = decode(encoded['nodes'], encoded['edges'], encoded['type']);
        expect(decoded, new isInstanceOf<CGraph>());
        expect(graph, equals(decoded));
      });
    });*/
  });
}

idCmp(x, y) {
  x = x['id'].toString();
  y = y['id'].toString();
  return (x as String).compareTo(y);
//  if (x > y) return 1;
//  if (x < y) return -1;
//  return 0;
}
